% Developed by Roger Skjetne

function [ omega, omega_dot, body_acc ] = discrete_diff( oqus )
%DISCRETE_DIFF_BODY_FRAME Discrete differentiation of position/orientation
%   The input is raw data from Qualisys, and the output is linear and
%   angular accelerations in body frame

%   Using low pass filtering and Euler differentiation
dt = 0.01;

%Translating all oqus angles from degrees to radians
oqus(5:7,:) = deg2rad(oqus(5:7,:));

%Remove measurement noise using low pass filter
med = median(oqus(2:7,1:100),2);
adjusted = [oqus(1,:); bsxfun(@minus, oqus(2:7,:), med)];
oqus_lp_filtered = lp_filter(adjusted, 100);
oqus_lp_filtered(2:7,:) = bsxfun(@plus, oqus_lp_filtered(2:7,:), med);

%Differentiate to get linear velocity and angular rate in {f}
omega = diff([oqus_lp_filtered(5:7,:)';oqus_lp_filtered(5:7,end)'])/dt;
vel = diff([oqus_lp_filtered(2:4,:)';oqus_lp_filtered(2:4,end)'])/dt;

%Rotate/translate velocities to {b}
for i = 1:length(omega)
    [J, J1, J2] = eulerang(oqus_lp_filtered(5,i), oqus_lp_filtered(6,i), oqus_lp_filtered(7,i));
    omega(i,:) = inv(J2)*omega(i,:)';
    vel(i,:) = inv(J1)*vel(i,:)';
end

%Differentiate to get accelerations in {b}
alpha = diff([omega; omega(end,:)])/dt;
acc = diff([vel; vel(end,:)])/dt;

%Pre-allocate output variables
body_acc = acc';
omega_dot = alpha';

%Insert time in first row, and remove the 2 last values
omega = omega(1:end-1,:)';
omega = [oqus(1,1:length(omega(1,:))); omega];
omega_dot(:,end-1:end) = [];
omega_dot = [oqus(1,1:length(omega_dot(1,:)));omega_dot];
body_acc(:,end-1:end) = [];
body_acc = [oqus(1,1:length(body_acc(1,:)));body_acc];
end