function [ etadot, etaddot, nu , delay, timevecout] = etadotFIR( etadata, timevec, dt )
% FIR DIFFERENTIATOR to calculate eta_dot and nu a marine vehicle. 
%   Based on Udjus, Guttorm NTNU MTech thesis (2017)
%   Roger Skjetne - 01.05.2020
%

%Filter properties
Nf = 50;        %Filter order
Fpass = 2;      %Passband frequency
Fstop = 5;      %Stopband frequency
Fs = 1/dt;      %Sampling rate

%Designing differentiator FIR filter
d = designfilt('differentiatorfir','FilterOrder',Nf,'PassbandFrequency',...
    Fpass,'StopbandFrequency',Fstop,'SampleRate',Fs);

[M,N] = size(etadata);

delay = mean(grpdelay(d));
v = zeros(3,N);
a = zeros(3,N);
for ii = 1:M
    v(ii,:) = filter(d, etadata(ii,:))/dt;
    a(ii,:) = filter(d,v(ii,:))/dt;
end

etadot  = v;
etaddot = a;
nu      = zeros(3,N);
for ii=1:N
    nu(:,ii) = rot_z(etadata(3,ii))'*v(:,ii);
end

% Brute force compensating of filter delay
timevecout = timevec(1:end-2*delay);
etadot(:,1:2*delay) = [];
etaddot(1:2*delay) = [];
nu(:,1:2*delay) = [];


end