function [ filtered_data ] = lp_filter( raw_data, Fs )
%Lowpass filter for raw IMU data
%   Input argument is the vector of measurements from IMU, usually 6xn
%   matrix with n measurements. Output is the filtered state, corrected for
%   time delay due to filtering. First line of input data is the time

%Pre-allocating output data
filtered_data = [];

%Lowpass filter properties
Fpass = 2;      %Passband frequency
Fstop = 5;      %Stopband frequency
Apass = 0.5;    %Passband attenuation
Astop = 100;    %Stopband ripple

%Designing equiripple lowpass FIR
df = designfilt('lowpassfir','StopbandFrequency',Fstop, ...
      'PassbandFrequency',Fpass,'StopbandAttenuation',Astop, ...
      'PassbandRipple',Apass,'SampleRate',Fs,'DesignMethod','equiripple');

%Estimating group delay according to filter properties
D = floor(mean(grpdelay(df)))+1;

%Correcting time by group delay
filtered_data(1,:) = raw_data(1,1:end-D);

%Filtering each variable
for i = 2:length(raw_data(:,1))
    temp = filter(df, raw_data(i,:));       %Filtering
    filtered_data(i,:) = temp(D+1:end);    %Correcting group delay
end

end