%close all;
clear all;

%% Constants
samp_time = 1/50;
deg2rad = pi/180;

%% Load data
[eta_meas_vec, drone_table_vec, nr_tests] = get_meas(deg2rad);

% Estimating velocty data
time_data      = cell(nr_tests,1);
nu_data_vec    = cell(nr_tests,1);
eta_dot_data   = cell(nr_tests,1);
for i = 1:nr_tests
    len = length(eta_meas_vec{i});
    time_vec = zeros(1,len);
    for ii = 1:len
        time_vec(ii)  = samp_time*(ii-1);
    end
    time_data{i} = time_vec;
    eta_data = eta_meas_vec{i}';
    [etadot,etaddot,nu,delay,time_vec_out] = etadotFIR(eta_data,time_vec,samp_time);
    nu_data_vec{i} = [time_vec_out; nu];
    eta_dot_data{i} = [time_vec_out; etadot];
end

%nu_data_vec [m/s,m/s,rad/s]

%% DFO      
TolFun = 10; %Termination tolerance
options = optimset('PlotFcns',@optimplotfval);
options = optimset(options, 'TolFun', TolFun, 'MaxIter', 400,...
    'MaxFunEvals', 1000, 'Display','notify');

D_optim_alt = 2; % 1: Linear damping; 2: Linear and quadratic damping

% Set seed
rng('shuffle');  % avoid reapeting random numbers
n =3;
fval_vec = zeros(n,1);
history_vec = {};

switch D_optim_alt
    case 1 
        nx = 4;
        % x =  Xu   Yv  Yr  Nr
        LB = [ 1,   50, -3,  1  ]; % Kun D_lin
        UB = [ 20, 100,  3, 10  ]; % Kun D_lin
    case 2
        nx = 11;
        % x =  Xu  Yv   Yr   Nr   Xuu  Yvv  Yrv Yvr Nrv Nvr Nrr
        LB = [  1,  50, -3,  0.1,  5,   25, -5, -5, -5, -5, 0.1]; % D_lin and D_nonlin
        UB = [ 20, 100,  3,  5,   20,  100,  5,  5,  5,  5, 10 ]; % D_linand D_nonlin
    otherwise
        nx = 4;
        % x =  Xu   Yv  Yr  Nr
        LB = [ 1,   50, -3,  1  ]; % Kun D_lin
        UB = [ 20, 100,  3, 10  ]; % Kun D_lin
end        
x_vec = zeros(n,nx); 

for i =1:n
    % Random initialization of damping: x0
    % x0 = [d11,d22,d23,d33];  x0 = [4.03, 6.22, 0, 0.067];  % p2alfa
    x0 = LB + (UB-LB).*rand([1,nx]);  
    [history, x_opt, fval_opt] = runfminsearch_sim(x0, eta_meas_vec, nu_data_vec,...
        drone_table_vec, samp_time, nr_tests, options, LB, UB);
    
    x_vec(i,:) = x_opt;
    fval_vec(i) = fval_opt; 
    history_vec{i} = history;

    format longg; 
    x_opt 
    fval_opt
    
    D_lin_opt = [x_opt(1)    0.0      0.0;
                  0.0     x_opt(2)  x_opt(3);
                  0.0     x_opt(3)  x_opt(4)];
    %disp(eig(D_lin_opt));
end

%% Find optimal parameters
i_opt = find(min(fval_vec));
x_opt = x_vec(i_opt,:);
fval_opt = min(fval_vec);

D_opt = [x_opt(1)    0.0      0.0;
           0.0     x_opt(2)  x_opt(3);
           0.0     x_opt(3)  x_opt(4)];
       
%% Run simulation again to see plots:
plot_sim

%disp(fval_vec-mean(fval_vec)); disp(i_opt); disp(x_opt);
%disp(eig(D_opt));


