#!/usr/bin/env python
import numpy as np 
import math
from config import mass_total, linear_damping_case1, linear_damping_case2, quadratic_damping,\
         coriolis_centr

from jacobian import jacobianFD

class Model:
    def __init__(self, sampT_obs):  
        #Model params
        self.mass_total = mass_total
        self.linear_damping = linear_damping_case1
        self.quadratic_damping_const = quadratic_damping
        self.coriolis_centr_const = coriolis_centr
        self.quadratic_damping = np.zeros((3,3), float)
        self.damping_total = self.linear_damping 
        self.coriolis_centr = np.zeros((3,3), float)

        # State-space model:
        self.A = np.zeros((9,9), float)
        self.B = np.zeros((9,3),float)
        self.B[3:6,0:3] = np.linalg.inv(mass_total)
        self.E = np.zeros((9,3),float)
        self.E[6:9] = np.eye(3)
        self.H = np.zeros((4,9),int)
        self.H[0:3,0:3] = np.eye(3)
        self.H[3,5] = 1
        self.phi = np.zeros((9,9),float)
        self.Gamma = sampT_obs* self.E

        #Other:
        self.trans_matrix = np.eye(3) #J(psi)
        self.par_derr = np.zeros((9,9),float)
    
    def update(self, x, sampT_obs, case_model, case_parder):
        if case_model == 2: #nonlinear damping + coriolis
            self.set_damping(x)
            self.set_coriolis(x)
        
        self.set_trans_matrix(x)
        self.set_A_matrix()
        
        if case_parder == 1:
            self.partial_der(x)
        else:
            dx = 1e-5
            self.par_derr = jacobianFD(x,dx)
        self.set_phi_matrix(sampT_obs)

    def set_trans_matrix(self, x):
        self.trans_matrix[0:2,0:2] = np.array([[math.cos(x[2]), -math.sin(x[2])],\
             [math.sin(x[2]), math.cos(x[2])]])

    def set_damping(self, x):
        self.linear_damping = linear_damping_case2
        self.quadratic_damping = np.matmul(self.quadratic_damping_const, abs(x[3:6]))
        self.damping_total = self.linear_damping + self.quadratic_damping

    def set_coriolis(self, x):
        self.coriolis_centr[0,2] = self.coriolis_centr_const[0,2]*x[4]
        self.coriolis_centr[1,2] = self.coriolis_centr_const[1,2]*x[3]
        self.coriolis_centr[2,0] = self.coriolis_centr_const[2,0]*x[4]
        self.coriolis_centr[2,1] = self.coriolis_centr_const[2,1]*x[3]
    
    def set_A_matrix(self):
        #self.set_trans_matrix(x)
        self.A[0:3,3:6] = self.trans_matrix
        self.A[3:6,3:6] = -np.matmul(np.linalg.inv(self.mass_total),\
           (self.damping_total + self.coriolis_centr))
        self.A[3:6,6:9] = np.linalg.inv(self.mass_total)

    def set_phi_matrix(self, sampT_obs):
        self.phi = np.eye(9) + sampT_obs*self.par_derr

    def partial_der(self, x):
        self.par_derr[0,2] = -(math.sin(x[2])*x[3] + math.cos(x[2])*x[4]) 
        self.par_derr[1,2] = math.cos(x[2])*x[3] - math.sin(x[2])*x[4]
        self.par_derr[0:3,3:6] = self.trans_matrix
        # del(-inv(M)*C(v)*v)/del(v))
        a1 = - np.matmul(np.linalg.inv(self.mass_total),(\
            np.array([[0, self.coriolis_centr_const[0,2]*x[5], self.coriolis_centr_const[0,2]*x[4]],\
            [self.coriolis_centr_const[1,2]*x[5], 0, self.coriolis_centr_const[1,2]*x[3]],\
            [(self.coriolis_centr_const[2,0] + self.coriolis_centr_const[2,1])*x[4],\
                (self.coriolis_centr_const[2,0] + self.coriolis_centr_const[2,1])*x[3], 0]])))
        #del(-inv(M)*D(v)*v)/del(v):
        a2 = - np.matmul(np.linalg.inv(self.mass_total), \
            (self.linear_damping + 2*self.quadratic_damping))
        self.par_derr[3:6,3:6] = a1 + a2
        self.par_derr[3:6,6:9] = np.linalg.inv(self.mass_total)



   

        