#!/usr/bin/env python

import rospy
from p2_drone.msg import ControlForce, StateEstimate, ObserverEstimate, Reference, DShotRef
from geometry_msgs.msg import Vector3
from std_msgs.msg import Float32
from write_csv import Write_csv
from data_class import Data


class NodeW2f:
    def __init__(self):
        self.write_csv = Write_csv()
        self.data = Data()

        # Subscribers:
        rospy.Subscriber("/controller/control_force", ControlForce, self.cb_controller)
        rospy.Subscriber("/observer/state_estimate", StateEstimate, self.cb_state_estimate)
        rospy.Subscriber("/observer_pos", ObserverEstimate, self.cb_observer_estimate)
        rospy.Subscriber("/guidance/reference", Reference, self.cb_reference)
        rospy.Subscriber("thrust_allocation/thrust_setpoint", DShotRef, self.cb_thrust_sp)
        rospy.Subscriber("aps_meas", Vector3, self.cb_aps_meas)
        rospy.Subscriber("/observer/attitude", Vector3, self.cb_attitude)


    def spin(self):
        rate = rospy.Rate(self.data.rate)
        now = rospy.get_rostime()
        self.write_csv.new_file(now)

        while not rospy.is_shutdown():
            now = rospy.get_rostime()
            self.write_csv.write_line(self.data, now)
            rate.sleep()
        self.write_csv.close_file()

    def cb_controller(self, msg):
        self.data.set_control(msg)

    def cb_state_estimate(self, msg):
        self.data.set_state_estimate(msg)
    
    def cb_observer_estimate(self, msg):
        self.data.set_observer_estimate(msg)
    
    def cb_reference(self, msg):
        self.data.reference = msg
    
    def cb_thrust_sp(self, msg):
        self.data.thrust_sp = msg
    
    def cb_aps_meas(self, msg):
        self.data.aps_meas = msg
    
    def cb_attitude(self,msg):
        self.data.attitude = msg


if __name__ == '__main__':
    rospy.init_node('node_w2f', anonymous=True)
    w2f_node = NodeW2f()
    w2f_node.spin()
